#
#       HBS REDUCTION SUB ROUTINE
#
#       initialization of global variables : hbsinit.cl
#
#       1998/03/05      K.Kawabata  Ver.1.00  
#       1998/06/14      K.Kawabata  Ver.2.00  unification
#       2000/02/29      K.Kawabata  Ver.3.00  for SITe CCD
#       2005/11/02      M.Isogai    Ver.4.00  for Andor CCD
#

#####################################
#                                   #
#         User Definitions          #
#                                   #
#####################################

#--- Directory Setting *** should be attatch '/' at the end ***

### Directory of C-program Definition Header File
  set _defhdir   = "/home/hbs/iraf/0510cal/"
### Directory of 2hole-Diaph. calib files
  set _caldir   = "/home/hbs/iraf/0510cal/"
### Directory of CL-scripts 
  set _clscript  = "/home/hbs/iraf/clscript/"
### Directory of C-programs
  set _progdir   = "/home/hbs/iraf/prog/"
### Directory of miscellaneous 
  set _etcdir    = "/home/hbs/iraf/etc/"
### Directory of tables of spectrophotometric standards
  set _stddir    = "/home/hbs/iraf/standard/"

#--- Run-time mode, Data Storage, etc.

### Display processing commands?
  set _fg_ver    = "1"  # 1:verbose, 2:little display
### If found old images which will be made next, Replace or Skip?
  set _fg_mode   = "1" # 1:replace, 2:skip, 3:query
### Delete the images of last process ? (Valid before 'hbsextract')
  set _fg_del1   = "2" # 1:erase old images, 2:keep
### Delete the images of last process ? (Valid at and after 'hbsextract')
  set _fg_del2   = "2" # 1:erase old images, 2:keep (hbsskysub-hbs1d)
### Dark Averaging Mode
  set _fg_dark   = "1" # 1:group, 2:object, 3:exposure time
### Query at Bad pixel correction
  set _fg_badpix = "1" # 1:query, 2:no query
### q/u calculation mode for 2HOLE diaphragm
  set _iqucalc   = "1" # 1:variable sky, 2:clear and stable sky
### Error Estimation
  set _fg_err    = "1"  # 1:mean error (sigma/sqrt(n)), 2:standard deviation

### Reduction Mode
  set _objcal    = "1"  # 1: object obs. 2: UP,SP,GT obs.


#############################################
#                                           #
#      Don't Modify under this line !!      #
#                                           #
#############################################

set _defhname  = "hbsdefine.h"           # C-program definition header
set _badcol    = "_etcdir$badcol_TI.dat" # bad column table file
set _badpix    = "_etcdir$badpix_TI.dat" # bad pixel table file
set _nullfile  = "_etcdir$null.dat"      # null file
set _features  = "HBS.feature.*"     # feature table (root file name) 
set _bintable  = "HBS.bintab.*"      # Binning factor table (root file name) 
set _featuren  = "_etcdir$HBS.feature.TIccd"   # feature table (normal)
set _featuret  = "_etcdir$HBS.feature.TIOband" # feature table (TiO bands)
set _featureb  = "_etcdir$HBS.feature.E-Bal"   # feature table (Balmer em.)
set _photmet   = "hbsphot.par"           # photometric parameters
set _logfile   = "HBS.log"               # log file

set _rootlist  = "HBS.list"              # Prime list of HBSred
set _darklist  = "HBS.DARK.list"         # Dark group list
set _flatlist  = "HBS.FLAT.list"         # Flat group list
set _phlist    = "HBS.phot.list"         # Photmetric standard group list
set _nposhwp   = "HBS.nposhwp"           # Number of used position angle
set _iqulist   = "HBS.iqu.list"          # Iqu-group list
set _grouplist = "HBS.ave.list"          # Iqu-group average list
set _avelog    = "HBS.ave.log"           # Log of Iqu-average
set _phlog     = "HBS.phot.log"          # log of hbsphcor
set _qlist     = ".q"                    # q-group list
set _ulist     = ".u"                    # u-group list
set _ilist     = ".i"                    # i-group list

set _fl1hl1   = "FLAT.1HL.1t"   # 1HOLE
set _fl1hlp   = "FLAT.1HL.pxs"  # 1HOLE pixsel sensitivity
set _fl2hl1   = "FLAT.2HL.1t"   # 2HOLE first(sky wide slit)
set _fl2hl2   = "FLAT.2HL.2t"   # 2HOLE second(sky narrow slit)
set _fl2hl3   = "FLAT.2HL.3t"   # 2HOLE second(UP-trail)
set _fl2hlp   = "FLAT.2HL.pxs"  # 2HOLE pixsel sensitivity
set _fl2s21   = "FLAT.2S2.1t"   # 2SLIT02 first
set _fl2s22   = "FLAT.2S2.1t"   # 2SLIT02 second (modified first)
set _fl2s2p   = "FLAT.2S2.pxs"  # 2SLIT02 pixel sensitivity

set _up2hlu   = "KAP.2HL.Ut"     # 2HOLE kappa = k1 / k2
set _up2hll   = "KAP.2HL.Lt"     # 2HOLE kappa = k3 / k4
set _up2s2u   = "KAP.2S2.Ut"     # 2SLIT02 kappa = k1 / k2
set _up2s2l   = "KAP.2S2.Lt"     # 2SLIT02 kappa = k3 / k4
set _sensTI5  = "SENS.TI5"       # Spectral Sensitivity function image
set _sensSITe5 = "SENS.SITe5"    # Spectral Sensitivity function image
set _sensAndor = "SENS.Andor"    # Spectral Sensitivity function image

set _crostk   = "crostk"        # Cross-talk ratio spectra (.1, .2, .3)
set _2ndord   = "secord.dat"    # Pure 1st and 2nd order spectra

set _1to1we   = "f1to1we.tbl"   # 1-to-1-fit table (wavelength-dir, ext)
set _1to1wo   = "f1to1wo.tbl"   # 1-to-1-fit table (wavelength-dir, ord)
set _1to1se   = "f1to1se.tbl"   # 1-to-1-fit table (slit-length-dir, ext)
set _1to1so   = "f1to1so.tbl"   # 1-to-1-fit table (slit-length-dir, ord)
set _1to1sw   = "f1to1sw.tbl"   # 1-to-1-fit table (wavelength data of lines)

set _gthead   = "gtcor."        # root name of GT calibration file
set _uphead   = "upcor."        # root name of UP calibration file
set _sphead   = "spave."        # root name of SP calibration file

set _xsftlog   = "HBS.xsft.log"  # Output logfile of hbsarrange
set _ysftlog   = "HBS.ysft.log"  # Output logfile of hbs1d

set _hotpix    = 5000.0         # hot-pixel lower limit (ADU)
set _rej_x1    = 5000.0         # Starting wavelength for average in rejection
set _rej_x2    = 7000.0         # End wavelength for average in rejection

package hbs

#--- entry of C-programs

task    $cryofits5 $cryofits4 $cryofits3 $hbsgcc = "$foreign"
task    $hbs1dsub $hbsshiftsub $hbscbin = "$foreign"
task    $hbstfx $hbstfy $hbshotpxtab = "$foreign"
task    $hbsiqupt $hbsipol $hbsconerrbin = "$foreign"
task    $hbs2ossub = "$foreign"
task    $sed = "$foreign"

#--- entry of CL-script files

task $hbscalrfits = _clscript$hbscalrfits.cl
task $hbsrfits = _clscript$hbsrfits.cl
task $hbswfits = _clscript$hbswfits.cl
task $hbsccmpl = _clscript$hbsccmpl.cl
task $hbsred = _clscript$hbsred.cl
task $hbsfconv = _clscript$hbsfconv.cl
task hbsdisplay = _clscript$hbsdisplay.cl
task $hbsreheader = _clscript$hbsreheader.cl
task $hbsgroup = _clscript$hbsgroup.cl
task $hbsparam = _clscript$hbsparam.cl
task $hbsdarkave = _clscript$hbsdarkave.cl
task $hbsdarksub = _clscript$hbsdarksub.cl
task $hbsdarkcor = _clscript$hbsdarkcor.cl
task $hbstrim = _clscript$hbstrim.cl
task $hbsbadpix = _clscript$hbsbadpix.cl

task hbscoord = _clscript$hbscoord.cl
task hbscoordl = _clscript$hbscoordl.cl

task $hbspixsens = _clscript$hbspixsens.cl
task $hbsextract = _clscript$hbsextract.cl
task hbsapext = _clscript$hbsapext.cl
task hbsshift = _clscript$hbsshift.cl
task $hbsskysub = _clscript$hbsskysub.cl
task $hbsarrange = _clscript$hbsarrange.cl
task $hbsflat2 = _clscript$hbsflat2.cl
task $hbs1d = _clscript$hbs1d.cl
task hbsap1d = _clscript$hbsap1d.cl
task $hbscntcor = _clscript$hbscntcor.cl
task $hbsbin = _clscript$hbsbin.cl
task $hbsiqugroup = _clscript$hbsiqugroup.cl
task $hbsiqumake = _clscript$hbsiqumake.cl
task $hbsiquave = _clscript$hbsiquave.cl
task hbsaverage = _clscript$hbsaverage.cl
task $hbsmktable = _clscript$hbsmktable.cl

task hbsscy = _clscript$hbsscy.cl
task hbsscx = _clscript$hbsscx.cl
task $hbsclean = _clscript$hbsclean.cl

task $hbsphred = _clscript$hbsphred.cl
task $hbsphgroup = _clscript$hbsphgroup.cl
task $hbsphfit1 = _clscript$hbsphfit1.cl
task $hbsphfit2 = _clscript$hbsphfit2.cl
task $hbsphfit3 = _clscript$hbsphfit3.cl
task $hbsphfit4 = _clscript$hbsphfit4.cl
task $hbsphfit5 = _clscript$hbsphfit5.cl
task $hbsphcor = _clscript$hbsphcor.cl
task hbsphsig = _clscript$hbsphsig.cl
task hbsphbin = _clscript$hbsphbin.cl

#-- add for AndorCCD
task $andorconv = _clscript$andorconv.cl
task $andorhconv = _clscript$andorhconv.cl

keep

imred
onedspec
generic
obsolete
rv
#twodspec
#apextract

keep

#--- Compile of C-programs & set PATH

hbsccmpl

#--- Rfits if *.imh not exist in the calibration directory

hbscalrfits

set _initproc = "DONE"

keep
