#
#       HBS REDUCTION SUB ROUTINE
#
#       average I, q and u spectrum (use HBS.list)
#
#       1997/03/31      K.Kawabata ver.0.10
#       1997/09/09      K.Kawabata ver.1.00  using HBS.list
#       1998/05/31      K.Kawabata ver.2.10  unification and e/o rejection
#       2000/02/29      K.Kawabata ver.3.00  for SITe-CCD
#
#
procedure hbsiquave

begin

  file listoflist, iqulist, grouplist
  string hbslist[250]
  string khead[250]
  string klist0, klist1, klist2, klist3
  string qlist[250], ulist[250], ilist[250], diaph[250], filter[250]
  int max_theta_n[250]
  int c, i, j, n, n_list
  int  n_allc, n_nextc
  string fn_group
  int x1, x2
  real l1, l2, la, lb

  file logfn
  string fn
  file tmp_fn, tmp_list
  file tmp_fn1, tmp_fn2, tmp_fn3, tmp_fn4, tmp_fn5
  file tmp_i1, tmp_i2, tmp_q, tmp_u
  file tmp_k0, tmp_k1, tmp_k2, tmp_k3
  file tmp_qi, tmp_ui
  file tmp_k0i, tmp_k1i, tmp_k2i, tmp_k3i
  int n_i, n_q, n_u
  int n_k0, n_k1, n_k2, n_k3
  file fn_i1ave, fn_i1sig, fn_i1pix, fn_i1rej
  file fn_i2ave, fn_i2sig, fn_i2pix, fn_i2rej
  file fn_qave,  fn_qsig,  fn_qpix,  fn_qrej
  file fn_uave,  fn_usig,  fn_upix,  fn_urej
  file fn_k0ave, fn_k0sig, fn_k0pix, fn_k0rej
  file fn_k1ave, fn_k1sig, fn_k1pix, fn_k1rej
  file fn_k2ave, fn_k2sig, fn_k2pix, fn_k2rej
  file fn_k3ave, fn_k3sig, fn_k3pix, fn_k3rej
  file fn_kn
  string fn_head, fn_uplow
  file binfile
  file weightlist
  string buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7, buf8
  real rsigma = 3.0

  file fn_cmnt
  string cmnt
  string cmntext = ".cmnt"
  string cmnt_qrej = "Rejected_in_q"
  string cmnt_urej = "Rejected_in_u"

  int flag # 0:lower 1:upper -1:other
  string prmfile
  int flag_err

#--- initial settings

  logfn = envget( "_logfile" )

  if( access( "hbs.tmp.time" ) == 1 )
    delete( "hbs.tmp.time", ver- )
  time >> hbs.tmp.time
  list = "hbs.tmp.time"
  if( fscan( list, buf, buf1, buf2 ) != 3 )
  {
    printf( "Error: time task -- reading\n" )
    bye
  }
  delete( "hbs.tmp.time", ver- )

  print( "" )
  printf( "%s %s\n", buf1, buf2 )
  print( "---------------------------" )
  print( " Averaging i, q, u spectra"
  print( "---------------------------" )
  print( "" )

  print( "", >> logfn )
  printf( "%s %s\n", buf1, buf2, >> logfn )
  print( "---------------------------", >> logfn )
  print( " Averaging i, q, u spectra", >> logfn )
  print( "---------------------------", >> logfn )
  print( "", >> logfn )

  listoflist = envget( "_rootlist" )
  iqulist = envget( "_iqulist" )
  grouplist = envget( "_grouplist" )
  l1 = int( envget( "_rej_x1" ) )
  l2 = int( envget( "_rej_x2" ) )
  flag_err = int( envget( "_fg_err" ) )

  tmp_list = mktemp( "hbs.tmp.list." )
  tmp_fn  = mktemp( "hbs.tmp.fn." )
  tmp_fn1 = mktemp( "hbs.tmp.fn1." )
  tmp_fn2 = mktemp( "hbs.tmp.fn2." )
  tmp_fn3 = mktemp( "hbs.tmp.fn3." )
  tmp_fn4 = mktemp( "hbs.tmp.fn4." )
  tmp_fn5 = mktemp( "hbs.tmp.fn5." )
  tmp_i1 = mktemp( "hbs.tmp.fni1." )
  tmp_i2 = mktemp( "hbs.tmp.fni2." )
  tmp_q = mktemp( "hbs.tmp.fnq." )
  tmp_u = mktemp( "hbs.tmp.fnu." )
  tmp_k0 = mktemp( "hbs.tmp.fnk0." )
  tmp_k1 = mktemp( "hbs.tmp.fnk1." )
  tmp_k2 = mktemp( "hbs.tmp.fnk2." )
  tmp_k3 = mktemp( "hbs.tmp.fnk3." )

#----- delete old images

  printf( "Now deleting old images...\n" )
  printf( "Now deleting old images...\n", >> logfn )
  imdelete( "*.ave.imh", ver- )
  imdelete( "*.sig.imh", ver- )

#----- checking list

  if( access( grouplist ) == 1 )
    delete( grouplist, ver- )

#----- Scanning the list file

  if( access( iqulist ) == 0 )
  {
    printf( "Cannot find '%s'   Please do hbsiqugroup\n", iqulist )
    printf( "Cannot find '%s'   Please do hbsiqugroup\n", iqulist, >> logfn )
    bye
  }

  n_list = 0
  list = iqulist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7, buf8 ) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "HBS" )
    {
      print( "The list-file ", iqulist, " is not produced by HBSiquGROUP.CL" )
      print( "The list-file ", iqulist, " is not produced by HBSiquGROUP.CL",\
             >> logfn )
      bye
    }
    n_list += 1
    hbslist[n_list] = buf1
    ilist[n_list] = buf2
    qlist[n_list] = buf3
    ulist[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    max_theta_n[n_list] = int( buf7 )
    khead[n_list] = buf8
  }

 #---- selection of images for average

  for( i=1; i<=n_list; i+=1 )
  {
    flag = 999
    n_i = 0
    n_q = 0
    n_u = 0
    n_k0 = 0
    n_k1 = 0
    n_k2 = 0
    n_k3 = 0
    if( access( tmp_i1 ) )
      delete( tmp_i1 )
    if( access( tmp_i2 ) )
      delete( tmp_i2 )
    if( access( tmp_q ) )
      delete( tmp_q )
    if( access( tmp_u ) )
      delete( tmp_u )
    if( access( tmp_k0 ) )
      delete( tmp_k0 )
    if( access( tmp_k1 ) )
      delete( tmp_k1 )
    if( access( tmp_k2 ) )
      delete( tmp_k2 )
    if( access( tmp_k3 ) )
      delete( tmp_k3 )

    fn_cmnt = hbslist[i]//cmntext
    cmnt = ""

    printf( "--- %s ---\n", hbslist[i] )
    printf( "--- %s ---\n", hbslist[i], >> logfn )

    prmfile = hbslist[i]//".prm"

    list = prmfile
    if( fscan( list, buf, buf, buf, buf, buf, fn_head, rsigma ) != 7 )
    {
      printf( "Read error in '%s'\n", prmfile )
      printf( "Read error in '%s'\n", prmfile, >> logfn )
      bye
    }
    if( fn_head == "NONE" )
    {
      printf( "Skipped..\n" )
      next
    }

    if( access( ilist[i] ) == 1 )
    {
      list = ilist[i]
      while( fscan( list, buf1, buf2, buf3, buf4 ) != EOF )
      {
        if( access( buf3//".imh" ) )
          printf( "%s\n", buf3, >> tmp_i1 )
        if( access( buf4//".imh" ) )
          printf( "%s\n", buf4, >> tmp_i2 )
        n_i += 1
      }
    }
    if( access( qlist[i] ) == 1 )
    {
      list = qlist[i]
      while( fscan( list, buf, buf1, buf2, buf3 ) != EOF )
      {
        if( access( buf3//".imh" ) )
          printf( "%s\n", buf3, >> tmp_q )
        n_q += 1
      }
      tmp_qi = qlist[i]//"w"
      if( access( tmp_qi ) == 0 ){
        printf( "Warning: weight list '%s' not found. Weighting avoided.\n", tmp_qi )
	tmp_qi = "none"
      }
    }
    if( access( ulist[i] ) == 1 )
    {
      list = ulist[i]
      while( fscan( list, buf, buf1, buf2, buf3 ) != EOF )
      {
        if( access( buf3//".imh" ) )
          printf( "%s\n", buf3, >> tmp_u )
        n_u += 1
      }
      tmp_ui = ulist[i]//"w"
      if( access( tmp_ui ) == 0 ){
        printf( "Warning: weight list '%s' not found. Weighting avoided.\n", tmp_ui )
	tmp_ui = "none"
      }
    }

    klist0 = khead[i]//"0"
    klist1 = khead[i]//"1"
    klist2 = khead[i]//"2"
    klist3 = khead[i]//"3"

    if( access( klist0 ) == 1 )
    {
      list = klist0
      while( fscan( list, buf, buf1 ) != EOF )
      {
        if( access( buf1//".imh" ) )
          printf( "%s\n", buf1, >> tmp_k0 )
        n_k0 += 1
      }
      tmp_k0i = klist0//"w"
      if( access( tmp_k0i ) == 0 ){
        printf( "Warning: weight list '%s' not found. Weighting avoided.\n", tmp_k0i )
	tmp_k0i = "none"
      }
    }
    if( access( klist1 ) == 1 )
    {
      list = klist1
      while( fscan( list, buf, buf1 ) != EOF )
      {
        if( access( buf1//".imh" ) )
          printf( "%s\n", buf1, >> tmp_k1 )
        n_k1 += 1
      }
      tmp_k1i = klist1//"w"
      if( access( tmp_k1i ) == 0 ){
        printf( "Warning: weight list '%s' not found. Weighting avoided.\n", tmp_k1i )
	tmp_k1i = "none"
      }
    }
    if( access( klist2 ) == 1 )
    {
      list = klist2
      while( fscan( list, buf, buf1 ) != EOF )
      {
        if( access( buf1//".imh" ) )
          printf( "%s\n", buf1, >> tmp_k2 )
        n_k2 += 1
      }
      tmp_k2i = klist2//"w"
      if( access( tmp_k2i ) == 0 ){
        printf( "Warning: weight list '%s' not found. Weighting avoided.\n", tmp_k2i )
	tmp_k2i = "none"
      }
    }
    if( access( klist3 ) == 1 )
    {
      list = klist3
      while( fscan( list, buf, buf1 ) != EOF )
      {
        if( access( buf1//".imh" ) )
          printf( "%s\n", buf1, >> tmp_k3 )
        n_k3 += 1
      }
      tmp_k3i = klist3//"w"
      if( access( tmp_k3i ) == 0 ){
        printf( "Warning: weight list '%s' not found. Weighting avoided.\n", tmp_k3i )
	tmp_k3i = "none"
      }
    }

#----- assign filename

    fn_i1ave = fn_head//"i1.ave"
    fn_i1sig = fn_head//"i1.sig"
    fn_i1pix = fn_head//"i1.pix"
    fn_i1rej = fn_head//"i1.rej"
    fn_i2ave = fn_head//"i2.ave"
    fn_i2sig = fn_head//"i2.sig"
    fn_i2pix = fn_head//"i2.pix"
    fn_i2rej = fn_head//"i2.rej"
    fn_qave = fn_head//"q.ave"
    fn_qsig = fn_head//"q.sig"
    fn_qpix = fn_head//"q.pix"
    fn_qrej = fn_head//"q.rej"
    fn_uave = fn_head//"u.ave"
    fn_usig = fn_head//"u.sig"
    fn_upix = fn_head//"u.pix"
    fn_urej = fn_head//"u.rej"
    fn_k0ave = fn_head//"k0.ave"
    fn_k0sig = fn_head//"k0.sig"
    fn_k0pix = fn_head//"k0.pix"
    fn_k0rej = fn_head//"k0.rej"
    fn_k1ave = fn_head//"k1.ave"
    fn_k1sig = fn_head//"k1.sig"
    fn_k1pix = fn_head//"k1.pix"
    fn_k1rej = fn_head//"k1.rej"
    fn_k2ave = fn_head//"k2.ave"
    fn_k2sig = fn_head//"k2.sig"
    fn_k2pix = fn_head//"k2.pix"
    fn_k2rej = fn_head//"k2.rej"
    fn_k3ave = fn_head//"k3.ave"
    fn_k3sig = fn_head//"k3.sig"
    fn_k3pix = fn_head//"k3.pix"
    fn_k3rej = fn_head//"k3.rej"
    fn_kn    = fn_head//"k.num"

#----- display grouped list and enter title

#  printf( "\n------ binned I images (%d) -----\n", n_i )
#  type( tmp_i1 )
#  printf( "\n------ original I images (%d) -----\n", n_i )
#  type( tmp_i2 )
#  printf( "\n------ q(=Q/I) images (%d) -----\n", n_q )
#  type( tmp_q )
#  printf( "\n------ u(=U/I) images (%d) -----\n", n_u )
#  type( tmp_u )
#  printf( "\n------ kappa(0) images (%d) -----\n", n_k0 )
#  type( tmp_k0 )
#  printf( "\n------ kappa(1) images (%d) -----\n", n_k1 )
#  type( tmp_k1 )
#  printf( "\n------ kappa(2) images (%d) -----\n", n_k2 )
#  type( tmp_k2 )
#  printf( "\n------ kappa(3) images (%d) -----\n", n_k3 )
#  type( tmp_k3 )

#----- binning region

    x1 = 0
    x2 = 0
    la = 0.0
    lb = 0.0
    c = 0
    binfile = hbslist[i]//".binew"
    if( access( binfile ) == 0 )
    {
      printf( "Binning table '%s' not found for this group.\n", binfile )
      printf( "Please do hbsparam & hbsbin, then do hbsiquave again.\n" )
      bye
    }
    list = binfile
    while( fscan( list, buf1, buf2 ) != EOF )
    {
      c += 1
      la = real( buf1 )
      if( la >= l1 && lb < l1 )
        x1 = c
      if( la > l2 && lb <= l2 )
        x2 = c
      lb = la
    }
    if( x1 == 0 || x2 == 0 )
    {
      printf( "Error in reading '%s'\n", hbslist[i]//".binew" )
      printf( "Error in reading '%s'\n", hbslist[i]//".binew", >> logfn )
      bye
    }

#----- i(binned)

    if( n_i > 0 )
    {
      printf( "Now averaging I1-images (No rejection )\n" )
      printf( "Now averaging I1-images (No rejection )\n",\
               rsigma, >> logfn )
#      hbsaverage( tmp_i1, "none", x1, x2, fn_i1ave, fn_i1sig, fn_i1pix, fn_i1rej, rsigma )
      hbsaverage( tmp_i1, "none", 0, 0, fn_i1ave, fn_i1sig, fn_i1pix, fn_i1rej, rsig )
      if( flag_err == 1 && access( fn_i1pix//".imh" ) == 1 )
      {
        imfunc( fn_i1pix, tmp_fn1, "sqrt", ver- )
        imarith( fn_i1sig, "/", tmp_fn1, fn_i1sig, pixtype="real",\
                  calctype="real", ver- )
        imdelete( tmp_fn1, ver- )
      }

      if( access( fn_i1rej ) == 1 )
      {
        printf( "These rejected images are recorded in %s\n", fn_i1rej )
        printf( "These rejected images are recorded in %s\n",\
                fn_i1rej, >> logfn )
        type( fn_i1rej )
        type( fn_i1rej, >> logfn )
      }
      delete( tmp_i1, ver- )
    }

#----- i(original)

    if( n_i > 0 )
    {
      printf( "Now averaging I2-images (No rejection )\n" )
      printf( "Now averaging I2-images (No rejection )\n",\
              rsigma, >> logfn )
      hbsaverage( tmp_i2, "none", 0, 0, fn_i2ave, fn_i2sig, fn_i2pix, fn_i2rej, rsigma )
      if( flag_err == 1 && access( fn_i2pix//".imh" ) == 1 ) 
      {
        imfunc( fn_i2pix, tmp_fn1, "sqrt", ver- )
        imarith( fn_i2sig, "/", tmp_fn1, fn_i2sig, pixtype="real",\
                  calctype="real", ver- )
        imdelete( tmp_fn1, ver- )
      }
      delete( tmp_i2, ver- )
    }


    if( access( tmp_list ) == 1 )
      delete( tmp_list, ver- )

#----- kappa-0

    if( n_k0 > 0 )
    {
      printf( "Now averaging kappa(0)-images (%3.1fsigma rejection )\n", rsigma )
      printf( "Now averaging kappa(0)-images (%3.1fsigma rejection )\n",\
              rsigma, >> logfn )
      hbsaverage( tmp_k0, tmp_k0i, x1, x2, fn_k0ave, fn_k0sig, fn_k0pix, fn_k0rej, rsigma )
      if( flag_err == 1 && access( fn_k0pix//".imh" ) == 1 )
      {
        imfunc( fn_k0pix, tmp_fn1, "sqrt", ver- )
        imarith( fn_k0sig, "/", tmp_fn1, fn_k0sig, pixtype="real",\
                  calctype="real", ver- )
        imdelete( tmp_fn1, ver- )
      }

      if( access( fn_k0rej ) == 1 )
      {
        printf( "These rejected images are recorded in %s\n", fn_k0rej )
        printf( "These rejected images are recorded in %s\n",\
                fn_k0rej, >> logfn )
        type( fn_k0rej )
        type( fn_k0rej, >> logfn )
      }

      delete( tmp_k0, ver- )
      if( access( fn_k0pix//".imh" ) == 1 )
        printf( "%s\n", fn_k0pix, >> tmp_list )
    }

#----- kappa-1

    if( n_k1 > 0 )
    {
      printf( "Now averaging kappa(1)-images (%3.1fsigma rejection )\n", rsigma )
      printf( "Now averaging kappa(1)-images (%3.1fsigma rejection )\n",\
              rsigma, >> logfn )
      hbsaverage( tmp_k1, tmp_k1i, x1, x2, fn_k1ave, fn_k1sig, fn_k1pix, fn_k1rej, rsigma )
      if( flag_err == 1 && access( fn_k1pix//".imh" ) == 1 )
      {
        imfunc( fn_k1pix, tmp_fn1, "sqrt", ver- )
        imarith( fn_k1sig, "/", tmp_fn1, fn_k1sig, pixtype="real",\
                  calctype="real", ver- )
        imdelete( tmp_fn1, ver- )
      }

      if( access( fn_k1rej ) == 1 )
      {
        printf( "These rejected images are recorded in %s\n", fn_k1rej )
        printf( "These rejected images are recorded in %s\n",\
                fn_k1rej, >> logfn )
        type( fn_k1rej )
        type( fn_k1rej, >> logfn )
      }

      delete( tmp_k1, ver- )
      if( access( fn_k1pix//".imh" ) == 1 )
        printf( "%s\n", fn_k1pix, >> tmp_list )
    }

#----- kappa-2

    if( n_k2 > 0 )
    {
      printf( "Now averaging kappa(2)-images (%3.1fsigma rejection )\n", rsigma )
      printf( "Now averaging kappa(2)-images (%3.1fsigma rejection )\n",\
              rsigma, >> logfn )
      hbsaverage( tmp_k2, tmp_k2i, x1, x2, fn_k2ave, fn_k2sig, fn_k2pix, fn_k2rej, rsigma )
      if( flag_err == 1 && access( fn_k2pix//".imh" ) == 1 )
      {
        imfunc( fn_k2pix, tmp_fn1, "sqrt", ver- )
        imarith( fn_k2sig, "/", tmp_fn1, fn_k2sig, pixtype="real",\
                  calctype="real", ver- )
        imdelete( tmp_fn1, ver- )
      }

      if( access( fn_k2rej ) == 1 )
      {
        printf( "These rejected images are recorded in %s\n", fn_k2rej )
        printf( "These rejected images are recorded in %s\n",\
                fn_k2rej, >> logfn )
        type( fn_k2rej )
        type( fn_k2rej, >> logfn )
      }

      delete( tmp_k2, ver- )
      if( access( fn_k2pix//".imh" ) == 1 )
        printf( "%s\n", fn_k2pix, >> tmp_list )
    }

#----- kappa-3

    if( n_k3 > 0 )
    {
      printf( "Now averaging kappa(3)-images (%3.1fsigma rejection )\n", rsigma )
      printf( "Now averaging kappa(3)-images (%3.1fsigma rejection )\n",\
              rsigma, >> logfn )
      hbsaverage( tmp_k3, tmp_k3i, x1, x2, fn_k3ave, fn_k3sig, fn_k3pix, fn_k3rej, rsigma )
      if( flag_err == 1 && access( fn_k3pix//".imh" ) == 1 )
      {
        imfunc( fn_k3pix, tmp_fn1, "sqrt", ver- )
        imarith( fn_k3sig, "/", tmp_fn1, fn_k3sig, pixtype="real",\
                  calctype="real", ver- )
        imdelete( tmp_fn1, ver- )
      }

      if( access( fn_k3rej ) == 1 )
      {
        printf( "These rejected images are recorded in %s\n", fn_k3rej )
        type( fn_k3rej )
      }

      delete( tmp_k3, ver- )
      if( access( fn_k3pix//".imh" ) == 1 )
        printf( "%s\n", fn_k3pix, >> tmp_list )
    }

#----- number of pixels to be used in averaging

    if( access( tmp_list ) == 1 )
    {
      if( access( fn_kn//".imh" ) == 1 )
        imdelete( fn_kn, ver- )
      imsum( input="@"//tmp_list, output=fn_kn, pixtype="real",\
              calctype="real", ver- )
      delete( tmp_list, ver- )
    }
    else
    {
      imcopy( fn_i2ave, fn_kn, ver- )
      imreplace( fn_kn, "0.0" )
    }

#----- q

    if( n_q > 0 )
    {
      printf( "Now averaging q-images (%3.1fsigma rejection )\n", rsigma )
      printf( "Now averaging q-images (%3.1fsigma rejection )\n",\
              rsigma, >> logfn )
      hbsaverage( tmp_q, tmp_qi, x1, x2, fn_qave, fn_qsig, fn_qpix, fn_qrej, rsigma )
#      if( flag_err == 1 && access( fn_qpix//".imh" ) == 1 )
#      {
#        imfunc( fn_qpix, tmp_fn1, "sqrt", ver- )
#        imarith( fn_qsig, "/", tmp_fn1, fn_qsig, pixtype="real",\
#                  calctype="real", ver- )
#        imdelete( tmp_fn1, ver- )
#      }

      if( access( fn_qrej ) == 1 )
      {
        printf( "These rejected images are recorded in %s\n", fn_qrej )
        printf( "These rejected images are recorded in %s\n",\
                fn_qrej, >> logfn )
        type( fn_qrej )
        type( fn_qrej, >> logfn )

	j = stridx( cmnt, substr( cmnt_qrej, 1, 1 ) )
	if( j > 0 )
	{
	  buf = substr( cmnt, j, strlen( cmnt ) )
	  while( strlen( buf ) > strlen( cmnt_qrej ) )
	  {
	    if( substr( buf, 1, strlen( cmnt_qrej ) ) == cmnt_qrej )
	      goto norecord1
	    buf = substr( buf, 2, strlen( buf ) )
	    j = stridx( buf, substr( cmnt_qrej, 1, 1 ) )
	    if( j == 0 )
	      break
	    buf = substr( buf, j, strlen( buf ) )
	  }
	}
	cmnt = cmnt//"_"//cmnt_qrej
norecord1:

      }
      if( access( fn_qsig//".imh" ) == 1 )
        imdelete( fn_qsig, ver- )

      if( n_k0 > 0 )
      {
        imfunc( fn_k0sig, tmp_fn1, "square", ver- )
        imarith( tmp_fn1, "/", fn_k0ave, tmp_fn1, pixtype="real",\
                  calctype="real", ver- )
        imarith( tmp_fn1, "/", fn_k0ave, tmp_fn1, pixtype="real",\
                  calctype="real", ver- )
      }
      else
      {
        imcopy( fn_i2ave, tmp_fn1, ver- )
        imreplace( tmp_fn1, 0 )
      }
      if( n_k2 > 0 )
      {
        imfunc( fn_k2sig, tmp_fn2, "square", ver- )
        imarith( tmp_fn2, "/", fn_k2ave, tmp_fn2, pixtype="real",\
                  calctype="real", ver- )
        imarith( tmp_fn2, "/", fn_k2ave, tmp_fn2, pixtype="real",\
                calctype="real", ver- )
      }
      else
      {
        imcopy( fn_i2ave, tmp_fn2, ver- )
        imreplace( tmp_fn2, 0 )
      }
      imarith( tmp_fn1, "+", tmp_fn2, tmp_fn1, pixtype="real",\
                calctype="real", ver- )
      imdelete( tmp_fn2, ver- )

      if( n_k0 > 0 && n_k2 > 0 )
      {
        imarith( fn_k0ave, "/", fn_k2ave, tmp_fn2, pixtype="real",\
                  calctype="real", ver- )
      }
      else
      {
        if( n_k0 > 0 )
        {
          imcopy( fn_k0ave, tmp_fn2, ver- )
        }
        else
        {
          imarith( 1.0, "/", fn_k2ave, tmp_fn2, pixtype="real",\
                    calctype="real", ver- )
        }
      }
      imfunc( tmp_fn2, tmp_fn3, "sqrt", ver- )
    #--- q
      imarith( 1, "+", tmp_fn3, tmp_fn4, pixtype="real",\
                calctype="real", ver- )
#      imarith( 1, "-", tmp_fn3, tmp_fn5, pixtype="real",\
#                calctype="real", ver- )
#      if( access( fn_qave//".imh" ) == 1 )
#        imdelete( fn_qave, ver- )
#      imarith( tmp_fn5, "/", tmp_fn4, fn_qave, pixtype="real",\
#                calctype="real", ver- )
    #--- q-sigma
#      imarith( tmp_fn3, "+", 1, tmp_fn3, pixtype="real",\
#               calctype="real", ver- )
      imdelete( tmp_fn3, ver- )
      imfunc( tmp_fn4, tmp_fn3, "square", ver- )
      imfunc( tmp_fn3, tmp_fn3, "square", ver- )
      imfunc( tmp_fn2, tmp_fn2, "abs", ver- )
      imarith( tmp_fn1, "*", tmp_fn2, tmp_fn1, pixtype="real",\
                calctype="real", ver- )
      imarith( tmp_fn1, "/", tmp_fn3, tmp_fn1, pixtype="real",\
                calctype="real", ver- )

      imfunc( tmp_fn1, fn_qsig, "sqrt", ver- )
      imdelete( tmp_fn1, ver- )
      imdelete( tmp_fn2, ver- )
      imdelete( tmp_fn3, ver- )
      imdelete( tmp_fn4, ver- )
#      imdelete( tmp_fn5, ver- )

      delete( tmp_q, ver- )

    }

#----- u

    if( n_u > 0 )
    {
      printf( "Now averaging u-images (%3.1fsigma rejection )\n", rsigma )
      printf( "Now averaging u-images (%3.1fsigma rejection )\n",\
              rsigma, >> logfn )
      hbsaverage( tmp_u, tmp_ui, x1, x2, fn_uave, fn_usig, fn_upix, fn_urej, rsigma )
#      if( flag_err == 1 && access( fn_upix//".imh" ) == 1 )
#      {
#        imfunc( fn_upix, tmp_fn1, "sqrt", ver- )
#        imarith( fn_usig, "/", tmp_fn1, fn_usig, pixtype="real",\
#                  calctype="real", ver- )
#        imdelete( tmp_fn1, ver- )
#      }

      if( access( fn_urej ) == 1 )
      {
        printf( "These rejected images are recorded in %s\n", fn_urej )
        printf( "These rejected images are recorded in %s\n",\
                fn_urej, >> logfn )
        type( fn_urej )
        type( fn_urej, >> logfn )

	j = stridx( cmnt, substr( cmnt_urej, 1, 1 ) )
	if( j > 0 )
	{
	  buf = substr( cmnt, j, strlen( cmnt ) )
	  while( strlen( buf ) > strlen( cmnt_urej ) )
	  {
	    if( substr( buf, 1, strlen( cmnt_urej ) ) == cmnt_urej )
	      goto norecord2
	    buf = substr( buf, 2, strlen( buf ) )
	    j = stridx( buf, substr( cmnt_urej, 1, 1 ) )
	    if( j == 0 )
	      break
	    buf = substr( buf, j, strlen( buf ) )
	  }
	}
	cmnt = cmnt//"_"//cmnt_urej
norecord2:

      }
      if( access( fn_usig//".imh" ) == 1 )
        imdelete( fn_usig, ver- )

      if( n_k1 > 0 )
      {
        imfunc( fn_k1sig, tmp_fn1, "square", ver- )
        imarith( tmp_fn1, "/", fn_k1ave, tmp_fn1, pixtype="real",\
                  calctype="real", ver- )
        imarith( tmp_fn1, "/", fn_k1ave, tmp_fn1, pixtype="real",\
                  calctype="real", ver- )
      }
      else
      {
        imcopy( fn_i2ave, tmp_fn1, ver- )
        imreplace( tmp_fn1, 0 )
      }
      if( n_k3 > 0 )
      {
        imfunc( fn_k3sig, tmp_fn2, "square", ver- )
        imarith( tmp_fn2, "/", fn_k3ave, tmp_fn2, pixtype="real",\
                  calctype="real", ver- )
        imarith( tmp_fn2, "/", fn_k3ave, tmp_fn2, pixtype="real",\
                  calctype="real", ver- )
      }
      else
      {
        imcopy( fn_i2ave, tmp_fn2, ver- )
        imreplace( tmp_fn2, 0 )
      }
      imarith( tmp_fn1, "+", tmp_fn2, tmp_fn1, pixtype="real",\
                calctype="real", ver- )
      imdelete( tmp_fn2, ver- )

      if( n_k1 > 0 && n_k3 > 0 )
      {
        imarith( fn_k1ave, "/", fn_k3ave, tmp_fn2, pixtype="real",\
                  calctype="real", ver- )
      }
      else
      {
        if( n_k1 > 0 )
        {
          imcopy( fn_k1ave, tmp_fn2, ver- )
        }
        else
        {
          imarith( 1.0, "/", fn_k3ave, tmp_fn2, pixtype="real",\
                    calctype="real", ver- )
        }
      }
      imfunc( tmp_fn2, tmp_fn3, "sqrt", ver- )
    #--- u
      imarith( 1, "+", tmp_fn3, tmp_fn4, pixtype="real",\
                calctype="real", ver- )
#      imarith( 1, "-", tmp_fn3, tmp_fn5, pixtype="real",\
#                calctype="real", ver- )
#      if( access( fn_uave//".imh" ) == 1 )
#        imdelete( fn_uave, ver- )
#      imarith( tmp_fn5, "/", tmp_fn4, fn_uave, pixtype="real",\
#                calctype="real", ver- )
    #--- u-sigma
#      imarith( tmp_fn3, "+", 1, tmp_fn3, pixtype="real",\
#               calctype="real", ver- )
      imdelete( tmp_fn3, ver- )
      imfunc( tmp_fn4, tmp_fn3, "square", ver- )
      imfunc( tmp_fn3, tmp_fn3, "square", ver- )
      imfunc( tmp_fn2, tmp_fn2, "abs", ver- )
      imarith( tmp_fn1, "*", tmp_fn2, tmp_fn1, pixtype="real",\
                calctype="real", ver- )
      imarith( tmp_fn1, "/", tmp_fn3, tmp_fn1, pixtype="real",\
                calctype="real", ver- )

      imfunc( tmp_fn1, fn_usig, "sqrt", ver- )
      imdelete( tmp_fn1, ver- )
      imdelete( tmp_fn2, ver- )
      imdelete( tmp_fn3, ver- )
      imdelete( tmp_fn4, ver- )
#      imdelete( tmp_fn5, ver- )

      delete( tmp_u, ver- )
    }

#----- record comment log files

    if( cmnt != "" )
    {
      if( access( fn_cmnt ) == 1 )
        delete( fn_cmnt, ver- )
      printf( "%s", cmnt, >> fn_cmnt )
    }

    printf( "\n" )
    printf( "\n", >> logfn )
    printf( "%s %s %s  %d %d %d  %s\n", fn_head, diaph[i], binfile,\
            n_i, n_q, n_u, fn_cmnt, >> grouplist )

  }

end
