#
#       HBS REDUCTION SUB ROUTINE
#
#       Modifying and re-producing flat-fielding images: hbsmakepix.cl
#
#       1997/04/08      K.Kawabata   ver 1.00
#       1997/09/03      K.Kawabata   ver 1.10  1hole diaphragm
#       1998/04/20      K.Kawabata   ver 2.00  unification
#       2005/04/07      M.Isogai     ver.2.10  for iraf v2.12
#
#

procedure hbsmakepix

begin

  int n_list, i, j, k
  string tmp_list
  string tmp_fn1, tmp_fn2
  string hbslist[50], hbsfile[50], diaph[50]
  int flattype[50], theta_n[50]
  string slit[50]
  file fn_tmp = "FLAT.tmp"
  file fn_pixsens
  int flatid
  file flatlist
  string buf="y"
  string buf1, buf2, buf3, buf4, buf5, buf6
  int xpix, ypix
  int nx
  int ny

    
  print( "" )
  print( "----------------------------------" )
  print( " Making a pixel sensitivity image" )
  print( "----------------------------------" )
  print( "" )

  tmp_list = mktemp( "hbs.tmp.list." )
  tmp_fn1 = mktemp( "hbs.tmp.image." )
  tmp_fn2 = mktemp( "hbs.tmp.image." )

  flatlist = envget( "_flatlist" )


#----- search FLAT list

  if( access( flatlist ) == 0 )
  {
    printf( "Cannot find %s. Do hbsflatgroup.\n", flatlist )
    bye
  }

#----- use existing FLAT.pixsens.tr ?

  if( access( fn_tmp//".imh" ) )
  {
    printf( "Detected old '%s' in this directory. Delete it ? (y/n) >> ",
            fn_tmp )
    k = scan( buf )
    while( buf != "y" && buf != "n" )
      k = scan( buf )
    if( buf == "n" )
      goto displayimage
    imdelete( fn_tmp, ver- )
  }

#----- Scanning this directory and searching FLAT images

scanlist:

  n_list = 0
  list = flatlist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6) != EOF )
  {
    if( substr( buf2, 1, 4 ) != "FLAT" )
    {
      print( flatlist//" is not produced by HBSFLATGROUP.CL" )
      next
    }
    n_list += 1
    hbslist[n_list] = buf1
    hbsfile[n_list] = buf2
    flattype[n_list] = int( buf3 )
    theta_n[n_list] = int( buf4 )
    diaph[n_list] = buf5
    slit[n_list] = buf6
  }

#----- print detected lists

  for( j=1; j<=5; j+=1 )
  {
    printf( "--- %d\n", j )
    for( i=1; i<=n_list; i+=1 )
    {
      if( flattype[i] == j )
      printf( "%s\n", hbslist[i] )
    }
  }

#----- Choice of image

  printf( "\n" )
  printf( "[1] 1       [2] 2      [3] 3      [4] 4\n" )
  printf( "[5] 1,2     [6] 2,3    [7] 3,4\n" )
  printf( "[8] 1,2,3   [9] 2,3,4  [10] 5\n" )
  printf( "[any other] 1,2,3,4\n" )
  printf( "\nPlease select combination of # (FLAT.#ti) >> " )

  buf = "0"
  k = scan( buf )
  while( k != 0 && k != 1 )
    k = scan( buf )

  k = int( buf )
  if( k < 1 || k > 10 )
    buf = "0"
  if( buf == "0" || buf == "1" || buf == "5" || buf == "8" )
  {
    for( i=1; i<=n_list; i+=1 )
    {
      if( flattype[i] == 1 )
      {
        type( hbslist[i], device="text", >> tmp_list )
        buf1 = diaph[i]
      }
    }
  }
  if( buf == "0" || buf == "2" || buf == "5" || buf == "6" ||\
      buf == "8" || buf == "9" )
  {
    for( i=1; i<=n_list; i+=1 )
    {
      if( flattype[i] == 2 )
      {
        type( hbslist[i], device="text", >> tmp_list )
        buf1 = diaph[i]
      }
    }
  }
  if( buf == "0" || buf == "3" || buf == "6" || buf == "7" ||\
      buf == "8" || buf == "9" )
  {
    for( i=1; i<=n_list; i+=1 )
    {
      if( flattype[i] == 3 )
      {
        type( hbslist[i], device="text", >> tmp_list )
        buf1 = diaph[i]
      }
    }
  }
  if( buf == "0" || buf == "4" || buf == "7" || buf == "9" )
  {
    for( i=1; i<=n_list; i+=1 )
    {
      if( flattype[i] == 4 )
      {
        type( hbslist[i], device="text", >> tmp_list )
        buf1 = diaph[i]
      }
    }
  }
  if( buf == "10" )
  {
    for( i=1; i<=n_list; i+=1 )
    {
      if( flattype[i] == 5 )
      {
        type( hbslist[i], device="text", >> tmp_list )
        buf1 = diaph[i]
      }
    }
  }


#----- flat type  

  buf = buf1
  fn_pixsens = ""

selectflat:

  if( substr( buf, 1, 3 ) == "1HL" )
  {
    fn_pixsens = envget( "_fl1hlp" )
    flatid = 1
  }
  if( substr( buf, 1, 3 ) == "2HL" )
  {
    fn_pixsens = envget( "_fl2hlp" )
    flatid = 2
  }
  if( substr( buf, 1, 3 ) == "2S2" )
  {
    fn_pixsens = envget( "_fl2s2p" )
    flatid = 3
  }

  if( fn_pixsens == "" )
  {
    printf( "Diaph type '%s' not valid.\n", buf )
    goto scanlist
  }
  printf( "Diaph:%s   FLAT-name:%s  OK? [y/n] >> ", buf, fn_pixsens )
  c = scan( buf )
  while( buf != "y" && buf != "n" )
    c = scan( buf )
  if( buf == "n" )
  {
    printf( "Select [1]:1HOLE [2]:2HOLES [3]:2SLIT02 >> " )
    c = scan( buf )
    while( buf != "1" && buf != "2" && buf != "3" )
      c = scan( buf )
    if( buf == "1" )
    {
      buf = "1HL"
      goto selectflat
    }
    else
    {
      if( buf == "2" )
      {
        buf = "2HL"
        goto selectflat
      }
      else
      {
        buf = "2S2"
        goto selectflat
      }
    }
  }
  if( access( fn_pixsens//".imh" ) == 1 )
  {
    printf( "Old '%s' found. Replace? [y/n] >> ", fn_pixsens )
    c = scan( buf )
    while( buf != "y" && buf != "n" )
      c = scan( buf )
    if( buf == "n" )
      goto selectflat
    imdelete( fn_pixsens, ver- )
  }

#----- average and extract

createimage :

  print(  "Now averaging selected images" )
#  oimcombine( "@"//tmp_list, tmp_fn1, combine="average",\
#             reject="none", project-, outtype="real",
#             offsets="none", masktype="none", scale="none",\
#             zero="none", weight="none", lthreshold=INDEF,\
#             hthreshold=INDEF, nlow=1, nhigh=1 )
  imsum( "@"//tmp_list, tmp_fn1, option="median",\
         pixtype="real", calctype="real", ver- )
  delete( tmp_list, ver- )

  print( "Now displaying averaged image" )
  display( tmp_fn1, 1 )

  print(  "Now extracting pixel-sensitivity image :"//fn_pixsens )

  imgets( tmp_fn1, "i_naxis1" )
  xpix = int( imgets.value )
  if( xpix > 800 )
  {
    if( flatid == 1 || flatid == 2 )
      nx = 5
    else
      nx = 3
  }
  else
  {
    if( flatid == 1 || flatid == 2 )
      nx = 5
    else
      nx = 1
  }
  imgets( tmp_fn1, "i_naxis2" )
  ypix = int( imgets.value )
  if( ypix > 800 )
    ny = 5
  else
    ny = 3

  boxcar( tmp_fn1, tmp_fn2, xwindow=nx, ywindow=ny, boundary="nearest" )
#  median( tmp_fn1, tmp_fn2, xwindow=nx, ywindow=ny, zloreject=INDEF,\
#          zhireject=INDEF, boundary="nearest", ver- )

  imarith( tmp_fn1, "/", tmp_fn2, fn_tmp, divzero=0.0,\
           hparams="", pixtype="real", calctype="real", ver- )
#  imreplace( fn_tmp, value=1.0, upper=0.85, lower=1.15 )
  imreplace( fn_tmp, value=1.0, upper=0.85 )
  imreplace( fn_tmp, value=1.0, lower=1.15 )
  imdelete( tmp_fn1, ver- )
  imdelete( tmp_fn2, ver- )

displayimage:

  printf( "Now displaying created '%s'\n", fn_tmp )
  display( fn_tmp, 1 )
  printf( "OK? [y]:yes, finish. [n]:create again >> " )
  k = scan( buf )
  while( buf != "y" && buf != "n" )
    k = scan( buf )
  if( buf == "n" )
  {
    imdelete( fn_tmp, ver- )
    goto scanlist
  }
  imrename( fn_tmp, fn_pixsens, ver- )

end
