#
#       HBS REDUCTION SUB ROUTINE
#
#      Prescan and make template four spectra : hbsprescan.cl
#
#       1997/03/21      K.Kawabata  Ver.0.10  for CRYOCAM manual reduc
#       1997/04/10      K.Kawabata  Ver.0.20  refer HBS.list
#       1997/08/01      K.Kawabata  Ver.0.30  for original fitting
#       2005/04/07      M.Isogai    Ver.0.40  for iraf v2.12
#


procedure hbspostscan


begin

  file fn_scan1 = "postscan.sk."
  string fn_scan[4]
  file fn_y4 = "hbspostscan.dat"
  string hbslist[50]
  string shorttyp[50], title[50]
  string exptime[50], diaph[50], filter[50]
  string slit[50]
  string tmp_list[4]
  string tmp_fn[4]
  string tmp_tbl[4]
  int c, i, j, n_list
  int nx, ny
  string buf = ""
  string buf1, buf2, buf3, buf4, buf5, buf6, buf7
  real slicelevel = 0.2
  real dx = 10
  real y[4], y1, y2, y3
  file listoflist = "HBS.list"


  for( i=1; i<=4; i+=1 )
  {
    tmp_list[i] = mktemp( "hbs.tmp.list." )
  }

  print( "" )
  print( "-----------------------------------------------" )
  print( " make a template of y-width of four spectra" )
  print( "-----------------------------------------------" )
  print( "" )


#---- confirmation

  for( i=1; i<=4; i+=1 )
  {
    fn_scan[i] = fn_scan1//i
    tmp_fn[i] = fn_scan1//"tmp."//i
    tmp_tbl[i] = fn_scan1//"tmp.tbl."//i

    if( access( tmp_fn[i]//".imh" ) )
      imdelete( tmp_fn[i], ver- )
    if( access( tmp_tbl[i] ) )
      delete( tmp_tbl[i], ver- )
  }

  if( access( fn_scan[1]//".imh" ) || access( fn_scan[2]//".imh" ) ||\
      access( fn_scan[3]//".imh" ) || access( fn_scan[4]//".imh" ) )
  {
    print( "Detected old '"//fn_scan[1]//"' or others. Use them ? (y/n)  >> " )
    c = scan( buf )
    while( buf != "y" && buf != "n" )
      c = scan( buf )
    if( buf == "y" )
      goto prescanning
    for( i=1; i<=4; i+=1 )
    {
      if( access( fn_scan[i]//".imh" ) )
      {
        imdelete( fn_scan[i], ver- )
      }
    }
  }

#----- Scanning the list file

  if( access( listoflist ) == 0 )
  {
    print( "Cannot find "//listoflist//"   Please do hbsgroup" )
    bye
  }
  n_list = 0
  list = listoflist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "HBS" )
    {
      print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL" )
      break
    }
    if( buf2 != "OBJ" || buf3 == "SKY" || buf3 == "sky" )
      next
    n_list += 1
    hbslist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7
  }

#---- selection of images for average

  for( i=1; i<=n_list; i+=1 )
  {
    printf( "%2d: %16s  %6s  %s  %s\n", i, hbslist[i], title[i],\
            diaph[i], filter[i] )
  }

nextstep1:
  printf( "Choose an UPPER-HOLE group  >> " )
  while( scan( buf ) != 1 ){}
  i = int( buf )
  if( i < 1 || i > n_list )
    goto nextstep1
  c = strlen( diaph[i] )
  if( substr( diaph[i], (c-4), c ) != "UPPER" )
  {
    printf( "%s is not using UPPER HOLE !!  avoided.\n", hbslist[i] )
    goto nextstep1
  }
  buf = tmp_list[1]
  sed( 's/\.fit\.imh/\.sk\.1\.imh/', hbslist[i], >> buf )
  buf = tmp_list[2]
  sed( 's/\.fit\.imh/\.sk\.2\.imh/', hbslist[i], >> buf )
  printf( "OK. %s is registed.  More ? [y/n]  >> ", hbslist[i] )
  i = scan( buf )
  while( buf != "y" && buf != "n" )
    i = scan( buf )
  if( buf == "y" )
    goto nextstep1

nextstep2:
  printf( "Choose a LOWER-HOLE group  >> " )
  while( scan( buf ) != 1 ){}
  i = int( buf )
  if( i < 1 || i > n_list )
    goto nextstep2
  c = strlen( diaph[i] )
  if( substr( diaph[i], (c-4), c ) != "LOWER" )
  {
    printf( "%s is not using LOWER HOLE !!  avoided.\n", hbslist[i] )
    goto nextstep2
  }
  buf = tmp_list[3]
  sed( 's/\.fit\.imh/\.sk\.3\.imh/', hbslist[i], >> buf )
  buf = tmp_list[4]
  sed( 's/\.fit\.imh/\.sk\.4\.imh/', hbslist[i], >> buf )
  printf( "OK. %s is registed.  More ? [y/n]  >> ", hbslist[i] )
  i = scan( buf )
  while( buf != "y" && buf != "n" )
    i = scan( buf )
  if( buf == "y" )
    goto nextstep2

#---- make reference image

  for( i=1; i<=4; i+=1 )
  {
    oimcombine( "@"//tmp_list[i], fn_scan[i], combine="average",\
               reject="none", project-, outtype="real",
               offsets="none", masktype="none", scale="none",\
               zero="none", weight="none", lthreshold=INDEF,\
               hthreshold=INDEF, nlow=1, nhigh=1 )
    delete( tmp_list[i], ver- )
  }

prescanning:

  for( i=1; i<=4; i+=1 )
  {
    boxcar( fn_scan[i], tmp_fn[i], dx, 1, boundary="nearest" )
  }

#---- search four spectra and define initial constants

  for( i=1; i<=4; i+=1 )
  {
    if( access( "database/id"//fn_scan[i] ) )
        delete( "database/id"//fn_scan[i] )
  }

  if( access( fn_y4 ) )
      delete( fn_y4 )

#---- scan four spectra

  imgets( fn_scan[1], 'i_naxis1' )
  nx = int( imgets.value )
  imgets( fn_scan[1], 'i_naxis2' )
  ny = int( imgets.value )

  for( i=1; i<=4; i+=1 )
  {
    if( access( fn_scan[i] ) == 1 )
      delete( fn_scan[i], ver- )
  }

  for( j=1; j<=8; j+=1 )
  {
     if( access( "hbspostscan."//j ) == 1 )
       delete( "hbspostscan."//j, ver- )
  }

  for( i=1; i<=4; i+=1 )
  {
    wfits( tmp_fn[i], tmp_tbl[i], bscale=1.0, bzero=0.0, long_header-,
         short_header-, bitpix=-32, scale-, autoscale- )

    hbsscan1( tmp_tbl[i], i, slicelevel, 80, nx )

  }

  for( i=1; i<=4; i+=1 )
  {
    delete( tmp_tbl[i], ver- )
    imdelete( tmp_fn[i], ver- )
  }

end
